/***************************************************************************
 *            NetixxxChat.c
 *
 *
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */



#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <arpa/inet.h>
#include "NetixxxChat.h"


/* COMMANDS TO IMPLEMENT:

		/me <something>
 */


static char *PS = "$> ";


/***
 *** shows program's usage
***/
void show_usage(int flag)
{
	if (flag==SHORT) /* short usage description */
	{
		printf("Usage:    NetixxxChat [-h | <options>] <host> <port>\n");
	}
	else if (flag==LONG) /* long usage description */
	{
		printf("Usage:    NetixxxChat [-h | <options>] <host> <port>\n");
		printf("    -h           shows this help\n");
		printf("    -l <file>    saves the conversation to <file>\n");
    	printf("    -n <nick>    sets the chatter's nickname\n");
    	printf("    <host>       IP address or hostname where NetixxxChat server is running\n");
		printf("    <port>       TCP port NetixxxChat server is listening on\n");
		show_credits();
	}
	else
	{
		exit(EXIT_FAILURE);
	}

	fflush(stdout);
}


/***
 *** ASCII logo
***/
void show_logo()
{
	printf("+---------------------+\n");
	printf("| NetixxxChat - client |\n");
	printf("+---------------------+\n");
	fflush(stdout);
}


/***
 *** shows input prompt
***/
void prompt(char *ps)
{
	printf(ps);
	fflush(stdout);
}


/***
 *** checks if the given host is a valid IPv4 address
 *** by searching the three dots in "xxx.xxx.xxx.xxx"
***/
boolean check_valid_ip(char *host)
{
	int i;
	int l = strlen(host);
	int dots = 0;

	for (i=0; i<l; i++)
		if (host[i]=='.')
			dots++;

	if (dots!=3)
		return FALSE;
	else
	{
		for (i=0; i<l; i++)
			if ((host[i]!='.')&&(isdigit(host[i])==0))
				return FALSE;

		return TRUE;
	}
}


/***
 *** executes the command that follows the '/' symbol
***/
/*
void exec_command(char *cmdstr)
{
	const char delimiters[] = " .,;:!-";
    char *token, *cp;
	int cmdlen = strlen(cmdstr);
	int i;
	int n = 0;

	cmdstr[cmdlen-1] = '\0';

	for (i=0; i<cmdlen; i++)
		if (cmdstr[i]==' ') n++;

    cp = cmdstr;
	for (i=0; i<=n; i++)
	{
		token = strtok(cp, delimiters);

		if (strcmp(token, "prompt")==0) ;
		cp = NULL;
	}
}*/


/***
 *** executes the command that follows the '/' symbol
***/
void exec_command(char *cmdstr)
{
	char *params[10]; /* OKKIO AL 10!!! */
	int cmdlen = strlen(cmdstr);
	int n = 0;
	int i;

	cmdstr[cmdlen-1] = '\0';
	n = split(cmdstr, params, 10);

	for (i=0; i<n; i++)
	{
		/*printf(":%s:\n", params[i]);*/

		if (strcmp(params[0], "date")==0)
		{
			printf("%s\n", getdateandtime());
			fflush(stdout);
		}
		else if (strcmp(params[0], "div")==0)
		{
			divisor();
		}
		else if (strcmp(params[0], "prompt")==0)
		{
			PS = params[1];
		}
		else if (strcmp(params[0], "quit")==0)
		{
			printf("Goodbye!\n");
			fflush(stdout);

			exit(EXIT_SUCCESS);
		}
		else
		{
			fprintf(stderr, "Wrong command!\n");
		}
	}
}


/***
 *** main
***/
int main(int argc, char *argv[])
{
	static unsigned int SWITCH = 1;
	boolean auth_rqst_sent = FALSE;
	boolean auth_permitted = FALSE;

	int s;
	int connection;
	int c;
	int i;

	char *remote_host;
	int remote_port;

	fd_set rfds;
	struct sockaddr_in myaddr;
	struct hostent *h;

	if ((argc==1)||(argc>6))
	{
		fprintf(stderr, "Wrong number of parameters.\n");
		show_usage(SHORT);

		return EXIT_FAILURE;
	}
	else if ((argc==2)&&(strcmp(argv[1], "-h")==0))
	{
		show_usage(LONG);

		return EXIT_SUCCESS;
	}
	else if ((argc-1)%2!=0) /* to avoid "NetixxxChat -n <host> <port>" */
	{
		fprintf(stderr, "Wrong number of parameters.\n");
		show_usage(SHORT);

		return EXIT_FAILURE;
	}
	else
	{
		char *username;
		char *nickname = NULL;
		char *tty;
		char *logsfile;

		for (i=1; i<argc; i++)
		{
			if (strcmp(argv[i], "-l")==0)
			{
				logsfile = argv[i+1];
			}
			else if (strcmp(argv[i], "-n")==0)
			{
				nickname = argv[i+1];
			}
		}

		if (nickname==NULL) nickname = "Guest"; /* sets the default nickname to "Guest" */
		username = getuserid();
		tty = ttyname(STDIN);

		remote_host = argv[argc-2];
		remote_port = atoi(argv[argc-1]);

		if ((h=gethostbyname(remote_host))==NULL)
		{
			herror("gethostbyname()");

			return EXIT_FAILURE;
		}
		else if ((remote_port<=0)||(remote_port>65536))
		{
			fprintf(stderr, "Wrong port number.\n");

			return EXIT_FAILURE;
		}
		else if ((s=socket(PF_INET, SOCK_STREAM, 0))==(-1))
		{
			perror("socket()");

			return EXIT_FAILURE;
		}
		else
		{
			myaddr.sin_family = AF_INET;
			myaddr.sin_addr.s_addr = ((struct in_addr *)h->h_addr_list[0])->s_addr;
			myaddr.sin_port = htons(remote_port);
			memset(&(myaddr.sin_zero), 0, sizeof(myaddr.sin_zero));

			if (check_valid_ip(remote_host)==FALSE)
			{
				struct in_addr in;
				memcpy(&in, h->h_addr_list[0], sizeof(in));
				remote_host = inet_ntoa(in);
			}

        	if (inet_pton(AF_INET, remote_host, &myaddr.sin_addr)<=0)
			{
				fprintf(stderr, "Wrong IP address format.\n");

				return EXIT_FAILURE;
			}
			else if ((connection=connect(s, (struct sockaddr *)&myaddr, sizeof(myaddr)))!=0)
			{
				perror("connect()");

				close(s);

				return EXIT_FAILURE;
			}
			else
			{
				struct message incmsg, *msgp_in;
				struct message outmsg, *msgp_out;

				msgp_in = &incmsg;
				msgp_out = &outmsg;

				show_logo();
				printf("Connection to %s estabilished on port %d\n", remote_host, remote_port);

				while (SWITCH)
				{
					if (auth_rqst_sent==FALSE) /* authorization request has *NOT* been sent yet (program runs for the first time :-) */
					{
						/* this builds the "authorization request" message */
						msgp_out->mtype = MSG_AUTH_RQST;
						strcpy(msgp_out->username, getuserid());
						strcpy(msgp_out->nickname, nickname);
						strcpy(msgp_out->tty, tty);
						strcpy(msgp_out->buffer, "");

						printf("Sending authorization request for user %s...\n", nickname);
						fflush(stdout);

						if (send(s, msgp_out, sizeof(struct message), 0)<0)
							perror("send()");

						auth_rqst_sent = TRUE;
					}
					else /* authorization request has been sent */
					{
						FD_ZERO(&rfds); /* resets file descriptors' set */
						FD_SET(s, &rfds); /* watches if socket has input (status changed)*/
						FD_SET(STDIN, &rfds); /* watches if stdin has input (status changed) */

						if (select(max(STDIN, s)+1, &rfds, NULL, NULL, NULL)==(-1))
						{
							perror("select()");

							return EXIT_FAILURE;
						}

						if (FD_ISSET(s, &rfds)) /* reads from socket */
						{
							fflush(stdout);

							c = recv(s, msgp_in, sizeof(struct message), 0); /* cisti al -1! */
							msgp_in->buffer[c] = '\0';

							if (c<=0)
							{
								if (c<0)
									perror("recv()");

								close(s);

								return EXIT_FAILURE;
							}
							else
							{
								if (msgp_in->mtype==MSG_AUTH_NO) /* connection refused */
								{
									printf("Sorry, access denied by server (cause: %s)\n", msgp_in->buffer);
									fflush(stdout);

									close(s);

									return EXIT_SUCCESS;
								}
								else if (msgp_in->mtype==MSG_AUTH_OK) /* connection accepted */
								{
									printf("Access granted for user %s on %s\n", nickname, getdateandtime());
									fflush(stdout);

									auth_permitted = TRUE;

									divisor();
								}
								else if (msgp_in->mtype==MSG_GENERIC) /* generic chat message */
								{
									if (strcmp(nickname, msgp_in->nickname)!=0)
									{
										printf("\n%s: %s", msgp_in->nickname, msgp_in->buffer);
										fflush(stdout);
									}
								}
								else if (msgp_in->mtype==MSG_SERVERNOTE) /* server notification message */
								{
									printf("<.NetixxxChat server (%s:%d)>: %s\n", remote_host, remote_port, msgp_in->buffer);
									fflush(stdout);
								}

								prompt(PS);
							}
						}

						if (FD_ISSET(STDIN, &rfds)) /* reads from STDIN */
						{
							if (auth_permitted==TRUE) /* authorization accepted (connection permitted) */
							{
								c = read(STDIN, msgp_out->buffer, BUFFER_SIZE);

								if (strlen(msgp_out->buffer)>=BUFFER_SIZE)
									msgp_out->buffer[BUFFER_SIZE-1] = '\0';
								else
									msgp_out->buffer[c] = '\0';

								if (msgp_out->buffer[0]=='/') /* checks for command */
								{
									char *cmdptr = msgp_out->buffer;
									cmdptr++; /* moves to first char after the '/' symbol */

									exec_command(cmdptr);

									prompt(PS);
								}
								else /* normal message */
								{
									msgp_out->mtype = MSG_GENERIC;
	
									if (c>0)
									{
										if (send(s, msgp_out, sizeof(struct message), 0)<0)
											perror("send()");
									}
									else
									{
										if (c<0)
											perror("read()");
	
										close(s);
	
										return EXIT_FAILURE;
									}
								}
							}
							else
							{
								printf("DEBUG: autorizzazione NON concessa\n"); /***/
								close(s);

								return EXIT_SUCCESS;
							}
						}
					}
				}

				return EXIT_SUCCESS;
			}
		}
	}
}
