/***************************************************************************
 *            NetixxxChat.h
 *
 *
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */



#ifndef _NetixxxChat_H
#define _NetixxxChat_H

#ifdef __cplusplus
extern "C"
{
#endif


/*** generic information ***/
#define		VERSION			"1.0 Beta"
#define		AUTHOR			"r00t"
#define		EMAIL			"r00t@bl4ckh0l3.com"
#define		PRJSITE			"wildfirez.com"

/*** useful special keys ***/
#define		CR				0x0d /* carriage return */
#define		ESC				0x1b /* escape (ESC) */
#define		TAB				0x09 /* tab */
#define		LF				0x0a /* line feed */
#define		BACKSPACE		0x08 /* backspace */

/*** ANSI colors ***/
#define		WHITE 			"\e[1;37m"
#define		LGRAY			"\e[0;37m"
#define		GRAY			"\e[1;30m"
#define		BLACK			"\e[0;30m"
#define		RED				"\e[0;31m"
#define		LRED			"\e[1;31m"
#define		GREEN			"\e[0;32m"
#define		LGREEN			"\e[1;32m"
#define		BROWN			"\e[0;33m"
#define		YELLOW			"\e[1;33m"
#define		BLUE			"\e[0;34m"
#define		LBLUE			"\e[1;34m"
#define		PURPLE			"\e[0;35m"
#define		PINK			"\e[1;35m"
#define		CYAN			"\e[0;36m"
#define		LCYAN			"\e[1;36m"
#define		NORMAL			"\e[0m"

/*** boolean values ***/
#define		FALSE			0
#define		TRUE			(!FALSE)

/*** macros ***/
#define		max(a, b)		(a>b)?a:b
#define		min(a, b)		(a<b)?a:b

/*** standard file descriptors ***/
#define		STDIN			0
#define		STDOUT			1
#define		STDERR			2

#define		SHORT			0
#define		LONG			1

#define		DEFAULT_PORT	5091
#define		DEFAULT_MAXC	10
#define		FD_OFFSET		4 /* file descriptors' offset */

#define		USERNAMEMLEN	16
#define		NICKNAMEMLEN	USERNAMEMLEN

#define		LOGINTIMELEN	25
#define		HOSTLEN			32 /* hostname max length */
#define		TTYMLEN			32 /* tty max length */

#define		BUFFER_SIZE		1025 /* inclusive of final '\0' */
#define		DIVLEN			48 /* divisor length */

#define		DEFAULT_TTWAIT	5 /* secs to wait before kicking a wrong client */

/*** message types ***/
#define		MSG_AUTH_RQST	0 /* authorization request */
#define		MSG_AUTH_OK		1 /* connection permitted */
#define		MSG_AUTH_NO		2 /* connection not permitted */
#define		MSG_GENERIC		3 /* generic chat message */
#define		MSG_SERVERNOTE	4 /* server notification message */


typedef int boolean;


void show_usage(int flag);
void show_logo();
void prompt(char *ps);

extern void show_credits();
extern char *getuserid();
extern char *getdateandtime();
extern void divisor();
extern int split(char *buf, char **argv, int max_args);


struct message
{
	int mtype; /* message type */
	char username[USERNAMEMLEN]; /* effective remote username */
	char nickname[NICKNAMEMLEN]; /* chatter's nickname */
	char tty[TTYMLEN]; /* remote user's tty */
	char buffer[BUFFER_SIZE]; /* text buffer */
};

struct user
{
	boolean freerecord; /* TRUE (free) or FALSE (occupied) */
	char login_time[LOGINTIMELEN]; /* login date and time */
	char username[USERNAMEMLEN]; /* user's remote username */
	char nickname[NICKNAMEMLEN]; /* user's nickname */
	char remote_host[HOSTLEN]; /* remote host */
	char remote_tty[TTYMLEN]; /* tty on remote host */
};


#ifdef __cplusplus
}
#endif

#endif /* _NetixxxChat_H */
