/***************************************************************************
 *            NetixxxChatd.c
 *
 *
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */


/* COOL NOTES:

		printf("Number: \'%0*d\'\n", 3, 20); // prints "020"

 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <arpa/inet.h>
#include "NetixxxChat.h"


static unsigned int SWITCH = 1;


/***
 *** shows program's usage
***/
void show_usage(int flag)
{
	if (flag==SHORT) /* short usage description */
	{
		printf("Usage:    NetixxxChat [-h] <options>\n");
	}
	else if (flag==LONG) /* long usage description */
	{
		printf("Usage:    NetixxxChat [-h] <options>\n");
		printf("    -h           shows this help\n");
		printf("    -k <file>    records all data from clients (such as a keylogger) to <file>\n");
		printf("    -l <file>    saves session logs to <file>\n");
		printf("    -m <num>     <num> is the maximum number of connections that can be\n");
		printf("                 accepted from clients (default is %d)\n", DEFAULT_MAXC);
		printf("    -p <port>    listens on TCP port <port> to accept connections\n");
		printf("                  (please note that not-root users can only open ports\n");
		printf("                  over TCP port 1023)\n");
		printf("    -r <a|b|l>   restricted access to the server\n");
		printf("                  (a = accept incoming connections from any address (default))\n");
		printf("                  (b = accept incoming connections according to blacklist)\n");
		printf("                  (l = accept incoming connections from localhost only)\n");
		printf("    -t <seconds> seconds before closing a connection (and kicking the client)\n");
		printf("    -v           enables server verbosity\n");
		show_credits();
	}
	else
	{
		exit(EXIT_FAILURE);
	}

	fflush(stdout);
}


/***
 *** ASCII logo
***/
void show_logo()
{
	printf("+---------------------+\n");
	printf("| NetixxxChat - server |\n");
	printf("+---------------------+\n");
	fflush(stdout);
}


/***
 *** sets all users' records free
***/
void setUserDBinitstate(struct user *udb, int size)
{
	int i;

	for (i=0; i<size; i++)
		udb[i].freerecord = TRUE; /* in the beginning all records were free...! */
}


/***
 *** sets new terminal properties for interactive input
***/
int SetNewIOS(struct termios oldtio, struct termios newtio)
{
	int rc = tcgetattr(0, &oldtio);

	if (rc)
	{
		perror("tcgetattr(0, &oldtio)");

		return 4;
	}

	memcpy(&newtio, &oldtio, sizeof(newtio));
	newtio.c_lflag &= ~(ISIG | ICANON | ECHO);
	newtio.c_iflag &= ~(INLCR | ICRNL | IXON | IXOFF);
	newtio.c_cflag &= ~(PARENB);
	newtio.c_cflag = (newtio.c_cflag & ~CSIZE) | CS8;
	newtio.c_cc[VMIN] = 1;

	rc = tcsetattr(0, TCSAFLUSH, &newtio);

	if (rc)
	{
		perror("tcsetattr(0, TCSAFLUSH, &newtio)");

		return 12;
	}
	else
		return rc;
}


/***
 *** restores old original terminal settings
***/
int RestoreOldIOS(struct termios oldtio)
{
	int rc = tcsetattr(0, TCSAFLUSH, &oldtio);

	if (rc)
	{
		perror("tcsetattr(0, TCSAFLUSH, &oldtio)");

		return 12;
	}
	else
		return rc;
}


/***
 *** reads chars from stdin without the need of pressing ENTER
***/
char interactive_read()
{
	int i;
	int rdlen;
	char c;
	char buf[12];
	char prbuff[128], *p;

	rdlen = read(0, buf, sizeof(buf));

	if (rdlen<0)
		/*perror("read(0)");*/
		return 0;
	else if (rdlen==0)
		return 1; /*leave = 1; // EOF */
	else
	{
		p = prbuff;

		for (i=0; i<rdlen; ++i)
		{
			c = (unsigned char)buf[i];

			if (c < ' ')
				p += sprintf(p, "^%c", c+64);
			else if (c==0x7f)
				p += sprintf(p, "^?");
			else
				p += sprintf(p, "%c", c);
		}
/*
		for (i=0; i<rdlen; ++i)
		{
			c = (unsigned char)buf[i];
			printf("%-16s %3d 0x%02x 0%03o\n", prbuff, c, c, c);
			prbuff[0] = '\0';
		}
*/
	    /*if (buf[0]==4) return 4; // ctrl-D */
		return buf[0];
	}
}


/***
 *** signal handler
***/
void getout(int SIGNUM)
{
	printf("Signal %d received...\n", SIGNUM);
	fflush(stdout);

#ifdef __sun__
    SWITCH = 0;
    signal(SIGNUM, getout);
#endif
#ifdef __linux__
    SWITCH = 0;
#endif
}


/***
 *** main
***/
int main(int argc, char *argv[])
{
	static unsigned int VERBOSE = 0; /* verbose mode on (1) or off (0) */
	static unsigned int SAVELOGS = 0; /* logs file enabled (1) or disabled (0) */
	static unsigned int SAVEDATA = 0; /* data file enabled (1) or disabled (0) */

	int s; /* listening socket for accept() (always in LISTEN status) */
	int fd;
	int fdnew; /* new connected socket (always in ESTABILISHED status if accept() returns fine) */
	int fdmax;
	int fdtemp;
	int i;

	int max_connections = DEFAULT_MAXC; /* maximum number of connections */
	int port = DEFAULT_PORT; /* port bound to the server */
	int users = 0; /* initially there are 0 users logged in */
	uint32_t incom_address = INADDR_ANY;

	fd_set master, read_fds; /* file descriptors' sets for network communications */
	struct sockaddr_in myaddr;
	struct sockaddr_in remoteaddr;

	/*struct termios oldtio, newtio;
	int oldrc, newrc;
	fd_set termset;*/

	/*time_t itime, ftime;*/ /* initial and final times for the authorization message waiting (useful to kick out clients such as FTP or SSH) */
	int time_to_wait = DEFAULT_TTWAIT;

	FILE *logsfile; /* file pointer to session logs file */
	FILE *datafile; /* file pointer clients' data file */
	char *logsfilename;
	char *datafilename;

	if ((argc==1)||(argc>13))
	{
		fprintf(stderr, "Wrong number of parameters.\n");
		show_usage(SHORT);

		return EXIT_FAILURE;
	}
	else if ((argc==2)&&(strcmp(argv[1], "-h")==0))
	{
		show_usage(LONG);

		return EXIT_SUCCESS;
	}
	else
	{
		struct message incmsg, *msgp_in;
		struct message outmsg, *msgp_out;
		struct user *UserDB;

		for (i=1; i<argc; i++)
		{
			if (strcmp(argv[i], "-k")==0)
			{
				datafilename = argv[i+1];

				if ((datafile=fopen(datafilename, "w+"))==NULL)
				{
					fprintf(stderr, "Error while accessing file \"%s\".\n", datafilename);

					return EXIT_FAILURE;
				}
				else
				{
					SAVEDATA = 1;
				}
			}
			else if (strcmp(argv[i], "-l")==0)
			{
				logsfilename = argv[i+1];

				if ((logsfile=fopen(logsfilename, "w+"))==NULL)
				{
					fprintf(stderr, "Error while accessing file \"%s\".\n", logsfilename);

					return EXIT_FAILURE;
				}
				else
				{
					SAVELOGS = 1;
				}
			}
			else if (strcmp(argv[i], "-m")==0)
			{
				max_connections = atoi(argv[i+1]);
			}
			else if (strcmp(argv[i], "-p")==0)
			{
				port = atoi(argv[i+1]);
			}
			else if (strcmp(argv[i], "-r")==0)
			{
				char opt = argv[i+1][0];

				switch (opt)
				{
					case 'a':
						incom_address = INADDR_ANY;
						break;
					case 'b':
						/* HANDLE BLACK LIST!!! */
						incom_address = INADDR_ANY;
						break;
					case 'l':
						incom_address = INADDR_LOOPBACK;
						break;
					default:
						fprintf(stderr, "Wrong parameter: \'%c\'\n", opt);
						exit(EXIT_FAILURE);
				}
			}
			else if (strcmp(argv[i], "-t")==0)
			{
				time_to_wait = atoi(argv[i+1]);
			}
			else if (strcmp(argv[i], "-v")==0)
			{
				VERBOSE = 1; /* sets verbose mode on */
			}
		}

		if ((s=socket(PF_INET, SOCK_STREAM, 0))==(-1))
		{
			perror("socket()");

			return EXIT_FAILURE;
		}

		myaddr.sin_family = AF_INET;
		myaddr.sin_addr.s_addr = htonl(incom_address);
		myaddr.sin_port = htons(port);
		memset(&(myaddr.sin_zero), 0, sizeof(myaddr.sin_zero));

		if (bind(s, (struct sockaddr *)&myaddr, sizeof(myaddr))==(-1))
		{
			perror("bind()");

			return EXIT_FAILURE;
		}

		if (listen(s, max_connections)==(-1))
		{
			perror("listen()");

			return EXIT_FAILURE;
		}

		UserDB = malloc(sizeof(struct user)*max_connections);
		msgp_in = &incmsg;
		msgp_out = &outmsg;

		/* setting initial state of the users database */
		setUserDBinitstate(UserDB, max_connections);

		FD_ZERO(&master);
		FD_SET(s, &master);
		fdmax = s; /* maximum file descriptor is the listening one for now */

		/*FD_ZERO(&termset);
		FD_SET(STDIN, &termset);*/

		if (VERBOSE)
		{
			show_logo();
			printf("NetixxxChat server is up and running!\n");
			printf("Maximum number of accepted connections: %d\n", max_connections);
			printf("Listening on port %d...\n", port);
			divisor();
			fflush(stdout);
		};

		if (signal(SIGUSR1, SIG_IGN)!=SIG_IGN) signal(SIGUSR1, getout); /* waits for signal SIGUSR1 */
		if (signal(SIGKILL, SIG_IGN)!=SIG_IGN) signal(SIGKILL, getout); /* waits for signal SIGKILL */
		if (signal(SIGINT, SIG_IGN)!=SIG_IGN) signal(SIGINT, getout); /* waits for signal SIGINT */
		if (signal(SIGTERM, SIG_IGN)!=SIG_IGN) signal(SIGTERM, getout); /* waits for signal SIGTERM */

		/*if (VERBOSE) printf("*** Setting terminal for interactive mode...\n");
		newrc = SetNewIOS(oldtio, newtio);*/

		while (SWITCH)
		{
			read_fds = master;

			/*if ((newrc==0)&&(FD_ISSET(0, &termset)))
			{
				char c = interactive_read();
				printf("Tasto premuto: %c (%d) (%x)\n", c, c, c);
				fflush(stdout);
			}*/

			if (select(fdmax+1, &read_fds, NULL, NULL, 0)==(-1)) /* timeval=0, not NULL */
			{
				perror("select()");

				return EXIT_FAILURE;
			}

			for (fd=0; fd<=fdmax; ++fd) /* ++fd */
			{
				if (FD_ISSET(fd, &read_fds))
				{
					if (fd==s)
					{
						socklen_t addrlen = sizeof(remoteaddr);

						if ((fdnew=accept(s, (struct sockaddr *)&remoteaddr,&addrlen))!=(-1))
						{
							if (fdnew>fdmax)
								fdmax = fdnew;

							if (VERBOSE) printf("[%d]: Connection from %s on %s\n", (fdmax-FD_OFFSET), inet_ntoa(remoteaddr.sin_addr), getdateandtime());

							FD_SET(fdnew, &master); /* adds new file descriptor to master set */
						}
						else
						{
							perror("accept()");
						}
					}
					else
					{
						/* e' qua che si blocca con FTP o SSH perchè dopo la connect non riceve nulla */
						int c = recv(fd, msgp_in, sizeof(struct message), 0);
						msgp_in->buffer[c] = '\0';

						if ((c>0)&&(*msgp_in->buffer!=EOF)) /* a message has been received */
						{
							for (fdtemp=0; fdtemp<=fdmax; ++fdtemp)
							{
								if (FD_ISSET(fdtemp, &master)&&(fdtemp!=s))
								{
									if (msgp_in->mtype==MSG_AUTH_RQST) /* received authorization request */
									{
										if (fdtemp==fd) /* checks if the message has been sent by the current fd */
										{
											if (VERBOSE) printf("[%d]: Authorization request from %s (%s@%s, %s)\n", (fdtemp-FD_OFFSET), msgp_in->nickname, msgp_in->username, inet_ntoa(remoteaddr.sin_addr), msgp_in->tty);

											if (users>=max_connections) /* number of users exceeded maximum limit */
											{
												char cause[] = "server is full.";

												/* builds the "authorization denied" message */
												msgp_out->mtype = MSG_AUTH_NO;
												strcpy(msgp_out->username, msgp_in->username);
												strcpy(msgp_out->nickname, msgp_in->nickname);
												strcpy(msgp_out->tty, msgp_in->tty);
												strcpy(msgp_out->buffer, cause);

												if (send(fdtemp, msgp_out, sizeof(struct message), 0)<0)
													perror("send()");

												if (VERBOSE) printf("[%d]: Authorization for user %s denied (cause: %s)\n", (fdtemp-FD_OFFSET), msgp_in->nickname, cause);

												close(fdtemp); /* ?!? */
												FD_CLR(fdtemp, &master); /* ?!? */
											}
											else /* user can be accepted */
											{
												if (UserDB[fdtemp-FD_OFFSET].freerecord==TRUE)
												{
													/* prepares a new entry in the users database */
													UserDB[fdtemp-FD_OFFSET].freerecord = FALSE; /* now the record is occupied by the new user */
													strcpy(UserDB[fdtemp-FD_OFFSET].login_time, getdateandtime()); /* login date and time */
													strcpy(UserDB[fdtemp-FD_OFFSET].username, msgp_in->username); /* user's remote username */
													strcpy(UserDB[fdtemp-FD_OFFSET].nickname, msgp_in->nickname); /* user's nickname */
													strcpy(UserDB[fdtemp-FD_OFFSET].remote_host, inet_ntoa(remoteaddr.sin_addr)); /* remote host */
													strcpy(UserDB[fdtemp-FD_OFFSET].remote_tty, msgp_in->tty); /* tty on remote host */

													/* builds the "authorization granted" message */
													msgp_out->mtype = MSG_AUTH_OK;
													strcpy(msgp_out->username, msgp_in->username);
													strcpy(msgp_out->nickname, msgp_in->nickname);
													strcpy(msgp_out->tty, msgp_in->tty);

													if (send(fdtemp, msgp_out, sizeof(struct message), 0)<0)
														perror("send()");

													users++; /* number of registered users increased by one */

													if (VERBOSE) printf("[%d]: Authorization for user %s granted!\n", (fdtemp-FD_OFFSET), msgp_in->nickname);
													if (VERBOSE) printf("Number of connected users: %d\n", users);
												}
												else
												{
													printf("DEBUG: Record %d occupato!\n", fdtemp-FD_OFFSET);
												}
											}
										}
									}
									else
									{
										if (msgp_in->mtype==MSG_GENERIC) /* received ordinary chat message */
										{
											msgp_out->mtype = MSG_GENERIC;
											strcpy(msgp_out->nickname, msgp_in->nickname);
											strcpy(msgp_out->buffer, msgp_in->buffer);

											if (send(fdtemp, msgp_out, sizeof(struct message), 0)<0)
												perror("send()");
										}
										else
										{
											if (fdtemp==fd) /* checks if the message has been sent by the current fd */
											{
												if (VERBOSE) printf("[%d]: WARNING - Invalid message from host %s\n", (fdtemp-FD_OFFSET), UserDB[fdtemp-FD_OFFSET].remote_host);

												close(fdtemp); /* ?!? */
												FD_CLR(fdtemp, &master); /* ?!? */
											}
										}
									}
								}
							}
						}
						else
						{
							if (c<0)
								perror("recv()");

							close(fd);
							FD_CLR(fd, &master);

							UserDB[fd-FD_OFFSET].freerecord = TRUE; /* now the occupied record is free */
							users = (users>0)?(users-1):users; /* number of registered users decreased by one */

							if (fd==fdmax)
								fdmax--; /* fdmax decreased by one if disconnected fd was the highest in the stack */

							if (VERBOSE) printf("[%d]: User %s disconnected on %s\n", (fd-FD_OFFSET), UserDB[fd-FD_OFFSET].nickname, getdateandtime());
							if (VERBOSE) printf("Number of connected users: %d\n", users);
						}
					}
				}
			}
		}

		free(UserDB); /* deallocates users database memory */
		if (VERBOSE) printf("Memory has been freed.\n");

		/*if (VERBOSE) printf("*** Restoring terminal to its initial state...\n");
		oldrc = RestoreOldIOS(oldtio);*/

		return EXIT_SUCCESS;
	}
}
